import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../../../tokens/proxy-path.token';
import { catchError, map, Observable, of } from 'rxjs';
import { GenericResponse, GenericResponseWrapper, GenericValidationError } from '../../../../generic-responses';
import {
    GrafanaAutogeneratedWidgetConfigRootResponse,
    LoadGrafanaDashboardsRootResponse
} from './grafana-autogenerated-widget.interface';

@Injectable({
    providedIn: 'root'
})
export class GrafanaAutogeneratedWidgetService {

    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }

    public loadWidgetConfig(widgetId: string): Observable<GrafanaAutogeneratedWidgetConfigRootResponse> {
        const proxyPath = this.proxyPath;


        return this.http.get<GrafanaAutogeneratedWidgetConfigRootResponse>(`${proxyPath}/grafana_module/grafana_configuration/grafanaWidget.json`, {
            params: {
                angular: true,
                widgetId: widgetId
            }
        }).pipe(
            map(data => {
                return data
            })
        )
    }

    public saveWidgetConfig(widgetId: string, host_id: number): Observable<GenericResponseWrapper> {
        const proxyPath = this.proxyPath;
        return this.http.post<any>(`${proxyPath}/grafana_module/grafana_configuration/grafanaWidget.json?angular=true`, {
            Widget: {
                id: widgetId
            },
            host_id: host_id
        })
            .pipe(
                map(data => {
                    // Return true on 200 Ok
                    return {
                        success: true,
                        data: data as GenericResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }

    public loadGrafanaDashboards(searchString: string): Observable<LoadGrafanaDashboardsRootResponse> {
        const proxyPath = this.proxyPath;

        return this.http.get<LoadGrafanaDashboardsRootResponse>(`${proxyPath}/grafana_module/grafana_configuration/getGrafanaDashboards.json`, {
            params: {
                'angular': true,
                'filter[Host.name]': searchString
            }
        }).pipe(
            map(data => {
                return data;
            })
        );
    }
}
