import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { ActionsButtonComponent } from '../../../components/actions-button/actions-button.component';
import {
    ActionsButtonElementComponent
} from '../../../components/actions-button-element/actions-button-element.component';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownDividerDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective
} from '@coreui/angular';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { DeleteAllModalComponent } from '../../../layouts/coreui/delete-all-modal/delete-all-modal.component';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { FormsModule } from '@angular/forms';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { MultiSelectComponent } from '../../../layouts/primeng/multi-select/multi-select/multi-select.component';
import { NgClass, NgForOf, NgIf } from '@angular/common';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import { TranslocoDirective, TranslocoPipe } from '@jsverse/transloco';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import {
    getDefaultServicetemplatesIndexParams,
    ServicetemplateIndex,
    ServicetemplateIndexRoot,
    ServicetemplatesIndexParams
} from '../../servicetemplates/servicetemplates.interface';
import { DeleteAllItem } from '../../../layouts/coreui/delete-all-modal/delete-all.interface';
import { Subscription } from 'rxjs';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { ServicetemplatesService } from '../servicetemplates.service';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { IndexPage } from '../../../pages.interface';

@Component({
    selector: 'oitc-servicetemplates-index',
    imports: [
        ActionsButtonComponent,
        ActionsButtonElementComponent,
        CardBodyComponent,
        CardComponent,
        CardFooterComponent,
        CardHeaderComponent,
        CardTitleDirective,
        ColComponent,
        ContainerComponent,
        DebounceDirective,
        DeleteAllModalComponent,
        DropdownDividerDirective,
        FaIconComponent,
        FormControlDirective,
        FormDirective,
        FormsModule,
        InputGroupComponent,
        InputGroupTextDirective,
        ItemSelectComponent,
        MatSort,
        MatSortHeader,
        MultiSelectComponent,
        NavComponent,
        NavItemComponent,
        NgForOf,
        NgIf,
        NoRecordsComponent,
        PaginateOrScrollComponent,
        PermissionDirective,
        RowComponent,
        SelectAllComponent,
        TableDirective,
        TranslocoDirective,
        TranslocoPipe,
        XsButtonDirective,
        NgClass,
        RouterLink,
        TableLoaderComponent
    ],
    templateUrl: './servicetemplates-index.component.html',
    styleUrl: './servicetemplates-index.component.css',
    providers: [
        { provide: DELETE_SERVICE_TOKEN, useClass: ServicetemplatesService } // Inject the ServicetemplatesService into the DeleteAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServicetemplatesIndexComponent implements OnInit, OnDestroy, IndexPage {

    public params: ServicetemplatesIndexParams = getDefaultServicetemplatesIndexParams();
    public servicetemplates?: ServicetemplateIndexRoot;
    public hideFilter: boolean = true;

    public servicetemplateTypes: any[] = [];
    public selectedItems: DeleteAllItem[] = [];

    private readonly ServicetemplatesService = inject(ServicetemplatesService);
    private subscriptions: Subscription = new Subscription();
    public readonly route = inject(ActivatedRoute);
    public readonly router = inject(Router);
    private SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly modalService = inject(ModalService);
    private cdr = inject(ChangeDetectorRef);


    public ngOnInit() {
        this.servicetemplateTypes = this.ServicetemplatesService.getServicetemplateTypes();

        this.subscriptions.add(this.route.queryParams.subscribe(params => {
            // Here, params is an object containing the current query parameters.
            // You can do something with these parameters here.
            //console.log(params);

            let id = params['id'];
            if (id) {
                this.params['filter[Servicetemplates.id][]'] = [].concat(id); // make sure we always get an array
            }

            let servicetemplatetype_id = params['servicetemplatetype_id'];
            if (servicetemplatetype_id) {
                // The Number cast is required otherwise the select box will not select the correct value
                if (Array.isArray(servicetemplatetype_id)) {
                    servicetemplatetype_id = servicetemplatetype_id.map(Number);
                } else {
                    servicetemplatetype_id = [Number(servicetemplatetype_id)];
                }

                this.params['filter[Servicetemplates.servicetemplatetype_id][]'] = servicetemplatetype_id;
            }


            this.loadServicetemplates();
        }));
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    public loadServicetemplates() {
        this.SelectionServiceService.deselectAll();

        this.subscriptions.add(this.ServicetemplatesService.getIndex(this.params)
            .subscribe((result) => {
                this.servicetemplates = result;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadServicetemplates();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadServicetemplates();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadServicetemplates();
    }

    public resetFilter() {
        this.params = getDefaultServicetemplatesIndexParams();
        this.loadServicetemplates();
    }

    // Open the Delete All Modal
    public toggleDeleteAllModal(servicetemplate?: ServicetemplateIndex) {
        let items: DeleteAllItem[] = [];

        if (servicetemplate) {
            // User just want to delete a single command
            items = [{
                id: servicetemplate.Servicetemplate.id,
                displayName: servicetemplate.Servicetemplate.name
            }];
        } else {
            // User clicked on delete selected button
            items = this.SelectionServiceService.getSelectedItems().map((item): DeleteAllItem => {
                return {
                    id: item.Servicetemplate.id,
                    displayName: item.Servicetemplate.name
                };
            });
        }

        // Pass selection to the modal
        this.selectedItems = items;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'deleteAllModal',
        });
    }

    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadServicetemplates();
        }
    }

    public navigateCopy() {
        let ids = this.SelectionServiceService.getSelectedItems().map(item => item.Servicetemplate.id).join(',');
        if (ids) {
            this.router.navigate(['/', 'servicetemplates', 'copy', ids]);
        }
    }
}
